@@ -63,13 +63,14 @@
     missing_docs,
     missing_debug_implementations,
     trivial_numeric_casts,
-    unstable_features,
     unused_import_braces,
     unused_qualifications
 )]
 #![cfg_attr(feature = "dev", allow(unstable_features))]
 #![cfg_attr(feature = "dev", feature(plugin))]
 #![cfg_attr(feature = "dev", plugin(clippy))]
+#![cfg_attr(not(feature = "nightly"), deny(unstable_features))]
+#![cfg_attr(feature = "nightly", feature(core_intrinsics))]
 #![recursion_limit="1500"]
 
 #[cfg(feature = "serde")]
