@@ -105,6 +105,7 @@ pub fn initialize() {
     };
 
     js! { @(no_return)
+        Module.STDWEB.noop = function() {};
         Module.STDWEB.to_js = function to_js( address ) {
             var kind = HEAPU8[ address + 12 ];
             if( kind === 0 ) {
@@ -144,7 +145,7 @@ pub fn initialize() {
                     output[ key ] = value;
                 }
                 return output;
-            } else if( kind === 9 || kind == 11 || kind == 12 ) {
+            } else if( kind === 9 || kind === 11 || kind === 12 ) {
                 return Module.STDWEB.acquire_js_reference( HEAP32[ address / 4 ] );
             } else if( kind === 10 ) {
                 var adapter_pointer = HEAPU32[ address / 4 ];
@@ -161,10 +162,41 @@ pub fn initialize() {
                 };
 
                 output.drop = function() {
-                    output.drop = null;
+                    output.drop = Module.STDWEB.noop;
                     Module.STDWEB.dyncall( "vi", deallocator_pointer, [pointer] );
                 };
 
+                return output;
+            } else if( kind === 13 ) {
+                var adapter_pointer = HEAPU32[ address / 4 ];
+                var pointer = HEAPU32[ (address + 4) / 4 ];
+                var deallocator_pointer = HEAPU32[ (address + 8) / 4 ];
+                var output = function() {
+                    if( pointer === 0 ) {
+                        throw new ReferenceError( "Already called or dropped FnOnce function called!" );
+                    }
+
+                    output.drop = Module.STDWEB.noop;
+                    var function_pointer = pointer;
+                    pointer = 0;
+
+                    var args = Module.STDWEB.alloc( 16 );
+                    Module.STDWEB.serialize_array( args, arguments );
+                    Module.STDWEB.dyncall( "vii", adapter_pointer, [function_pointer, args] );
+                    var result = Module.STDWEB.tmp;
+                    Module.STDWEB.tmp = null;
+
+                    return result;
+                };
+
+                output.drop = function() {
+                    output.drop = Module.STDWEB.noop;
+                    var function_pointer = pointer;
+                    pointer = 0;
+
+                    Module.STDWEB.dyncall( "vi", deallocator_pointer, [function_pointer] );
+                };
+
                 return output;
             }
         };
