@@ -14,6 +14,7 @@ use webcore::try_from::{TryFrom, TryInto};
 use webcore::number::Number;
 use webcore::object::Object;
 use webcore::array::Array;
+use webcore::type_name::type_name;
 
 use webcore::value::{
     Null,
@@ -952,7 +953,11 @@ macro_rules! impl_for_fn {
                     let $kind = match arguments.next().unwrap().try_into() {
                         Ok( value ) => value,
                         Err( _ ) => {
-                            panic!( "Argument #{} is not convertible", nth_argument + 1 );
+                            panic!(
+                                "Argument #{} is not convertible to '{}'",
+                                nth_argument + 1,
+                                type_name::< $kind >()
+                            );
                         }
                     };
 
@@ -1012,7 +1017,11 @@ macro_rules! impl_for_fn {
                     let $kind = match arguments.next().unwrap().try_into() {
                         Ok( value ) => value,
                         Err( _ ) => {
-                            panic!( "Argument #{} is not convertible", nth_argument + 1 );
+                            panic!(
+                                "Argument #{} is not convertible to '{}'",
+                                nth_argument + 1,
+                                type_name::< $kind >()
+                            );
                         }
                     };
 
@@ -1602,6 +1611,26 @@ mod test_serialization {
             1 + 2 + 3 + 4 + 5 + 6 + 7 + 8 + 9 + 10 + 11 + 12 + 13 + 14 + 15 + 16
         );
     }
+
+    #[test]
+    fn interpolated_args_are_converted_at_the_start() {
+        let mut string = "1".to_owned();
+        let callback = js! {
+            return function() {
+                return @{&string};
+            }
+        };
+
+        unsafe {
+            string.as_bytes_mut()[0] = b'2';
+        }
+
+        let result = js! {
+            return @{callback}();
+        };
+
+        assert_eq!( result, "1" );
+    }
 }
 
 #[cfg(test)]
