@@ -2,8 +2,12 @@ use std::collections::{BTreeMap, HashMap};
 use std::hash::Hash;
 use std::fmt;
 use std::error;
+use webcore::void::Void;
 use webcore::try_from::{TryFrom, TryInto};
 use webcore::number::{self, Number};
+use webcore::object::Object;
+use webcore::array::Array;
+use webcore::serialization::JsSerializable;
 
 /// A unit type representing JavaScript's `undefined`.
 #[derive(Copy, Clone, PartialEq, Eq, PartialOrd, Ord, Hash, Default, Debug)]
@@ -22,7 +26,7 @@ impl Reference {
     #[doc(hidden)]
     #[inline]
     pub unsafe fn from_raw_unchecked( refid: i32 ) -> Reference {
-        em_asm_int!( "Module.STDWEB.increment_refcount( $0 );", refid );
+        __js_raw_asm!( "Module.STDWEB.increment_refcount( $0 );", refid );
         Reference( refid )
     }
 
@@ -52,7 +56,7 @@ impl Clone for Reference {
 impl Drop for Reference {
     #[inline]
     fn drop( &mut self ) {
-        em_asm_int!( "Module.STDWEB.decrement_refcount( $0 );", self.0 );
+        __js_raw_asm!( "Module.STDWEB.decrement_refcount( $0 );", self.0 );
     }
 }
 
@@ -150,8 +154,8 @@ pub enum Value {
     Bool( bool ),
     Number( Number ),
     String( String ),
-    Array( Vec< Value > ),
-    Object( BTreeMap< String, Value > ), // TODO: Use our own type instead of using BTreeMap directly.
+    Array( Array ),
+    Object( Object ),
     Reference( Reference )
 }
 
@@ -166,6 +170,26 @@ impl Value {
         }
     }
 
+    /// Checks whenever the Value is of the Object variant.
+    #[inline]
+    pub fn is_object( &self ) -> bool {
+        if let Value::Object( _ ) = *self {
+            true
+        } else {
+            false
+        }
+    }
+
+    /// Checks whenever the Value is of the Array variant.
+    #[inline]
+    pub fn is_array( &self ) -> bool {
+        if let Value::Array( _ ) = *self {
+            true
+        } else {
+            false
+        }
+    }
+
     /// Gets a reference to the [Reference](struct.Reference.html) inside this `Value`.
     #[inline]
     pub fn as_reference( &self ) -> Option< &Reference > {
@@ -175,6 +199,24 @@ impl Value {
         }
     }
 
+    /// Gets a reference to the [Object](struct.Object.html) inside this `Value`.
+    #[inline]
+    pub fn as_object( &self ) -> Option< &Object > {
+        match *self {
+            Value::Object( ref object ) => Some( object ),
+            _ => None
+        }
+    }
+
+    /// Gets a reference to the [Array](struct.Array.html) inside this `Value`.
+    #[inline]
+    pub fn as_array( &self ) -> Option< &Array > {
+        match *self {
+            Value::Array( ref array ) => Some( array ),
+            _ => None
+        }
+    }
+
     /// Returns the [Reference](struct.Reference.html) inside this `Value`.
     #[inline]
     pub fn into_reference( self ) -> Option< Reference > {
@@ -184,6 +226,24 @@ impl Value {
         }
     }
 
+    /// Returns the [Object](struct.Object.html) inside this `Value`.
+    #[inline]
+    pub fn into_object( self ) -> Option< Object > {
+        match self {
+            Value::Object( object ) => Some( object ),
+            _ => None
+        }
+    }
+
+    /// Returns the [Array](struct.Array.html) inside this `Value`.
+    #[inline]
+    pub fn into_array( self ) -> Option< Array > {
+        match self {
+            Value::Array( array ) => Some( array ),
+            _ => None
+        }
+    }
+
     /// Converts a [Reference](struct.Reference.html) inside this `Value` into
     /// the given type `T`; doesn't check whenever the reference is really of type `T`.
     ///
@@ -340,89 +400,87 @@ impl< 'a > From< &'a mut char > for Value {
     }
 }
 
-impl< T: Into< Value > > From< Vec< T > > for Value {
+impl< T > From< Vec< T > > for Value where T: JsSerializable {
     #[inline]
     fn from( value: Vec< T > ) -> Self {
-        Value::Array( value.into_iter().map( |element| element.into() ).collect() )
+        value[..].into()
     }
 }
 
-impl< 'a, T > From< &'a Vec< T > > for Value where &'a T: Into< Value > {
+impl< 'a, T > From< &'a Vec< T > > for Value where T: JsSerializable {
     #[inline]
     fn from( value: &'a Vec< T > ) -> Self {
         value[..].into()
     }
 }
 
-impl< 'a, T > From< &'a mut Vec< T > > for Value where &'a T: Into< Value > {
+impl< 'a, T > From< &'a mut Vec< T > > for Value where T: JsSerializable {
     #[inline]
     fn from( value: &'a mut Vec< T > ) -> Self {
         value[..].into()
     }
 }
 
-impl< 'a, T > From< &'a [T] > for Value where &'a T: Into< Value > {
+impl< 'a, T > From< &'a [T] > for Value where T: JsSerializable {
     #[inline]
     fn from( value: &'a [T] ) -> Self {
-        Value::Array( value.iter().map( |element| {
-            element.into()
-        }).collect() )
+        let array: Array = value.into();
+        Value::Array( array )
     }
 }
 
-impl< 'a, T > From< &'a mut [T] > for Value where &'a T: Into< Value > {
+impl< 'a, T > From< &'a mut [T] > for Value where T: JsSerializable {
     #[inline]
     fn from( value: &'a mut [T] ) -> Self {
         (value as &[T]).into()
     }
 }
 
-// TODO: It would be nice to specialize this for values which are already of type Value.
-impl< K: Into< String >, V: Into< Value > > From< BTreeMap< K, V > > for Value {
+impl< K, V > From< BTreeMap< K, V > > for Value where K: AsRef< str >, V: JsSerializable {
     #[inline]
     fn from( value: BTreeMap< K, V > ) -> Self {
-        let value = value.into_iter().map( |(key, value)| (key.into(), value.into()) ).collect();
-        Value::Object( value )
+        let object: Object = value.into();
+        Value::Object( object )
     }
 }
 
-impl< 'a, K, V > From< &'a BTreeMap< K, V > > for Value where &'a K: Into< String >, &'a V: Into< Value > {
+impl< 'a, K, V > From< &'a BTreeMap< K, V > > for Value where K: AsRef< str >, V: JsSerializable {
     #[inline]
     fn from( value: &'a BTreeMap< K, V > ) -> Self {
-        let value = value.iter().map( |(key, value)| (key.into(), value.into()) ).collect();
-        Value::Object( value )
+        let object: Object = value.into();
+        Value::Object( object )
     }
 }
 
-impl< 'a, K, V > From< &'a mut BTreeMap< K, V > > for Value where &'a K: Into< String >, &'a V: Into< Value > {
+impl< 'a, K, V > From< &'a mut BTreeMap< K, V > > for Value where K: AsRef< str >, V: JsSerializable {
     #[inline]
     fn from( value: &'a mut BTreeMap< K, V > ) -> Self {
-        let value: &BTreeMap< K, V > = value;
-        value.into()
+        let object: Object = value.into();
+        Value::Object( object )
     }
 }
 
-impl< K: Into< String > + Hash + Eq, V: Into< Value > > From< HashMap< K, V > > for Value {
+impl< K, V > From< HashMap< K, V > > for Value where K: AsRef< str > + Eq + Hash, V: JsSerializable {
     #[inline]
     fn from( value: HashMap< K, V > ) -> Self {
-        let value = value.into_iter().map( |(key, value)| (key.into(), value.into()) ).collect();
-        Value::Object( value )
+        let object: Object = value.into();
+        Value::Object( object )
     }
 }
 
-impl< 'a, K: Hash + Eq, V > From< &'a HashMap< K, V > > for Value where &'a K: Into< String >, &'a V: Into< Value > {
+impl< 'a, K, V > From< &'a HashMap< K, V > > for Value where K: AsRef< str > + Eq + Hash, V: JsSerializable {
     #[inline]
     fn from( value: &'a HashMap< K, V > ) -> Self {
-        let value = value.iter().map( |(key, value)| (key.into(), value.into()) ).collect();
-        Value::Object( value )
+        let object: Object = value.into();
+        Value::Object( object )
     }
 }
 
-impl< 'a, K: Hash + Eq, V > From< &'a mut HashMap< K, V > > for Value where &'a K: Into< String >, &'a V: Into< Value > {
+impl< 'a, K, V > From< &'a mut HashMap< K, V > > for Value where K: AsRef< str > + Eq + Hash, V: JsSerializable {
     #[inline]
     fn from( value: &'a mut HashMap< K, V > ) -> Self {
-        let value: &HashMap< K, V > = value;
-        value.into()
+        let object: Object = value.into();
+        Value::Object( object )
     }
 }
 
@@ -478,6 +536,7 @@ macro_rules! impl_from_number {
 
 impl_from_number!( i8 i16 i32 u8 u16 u32 f32 f64 );
 impl_infallible_try_from! {
+    Value => Value;
     Undefined => Value;
     impl< 'a > for &'a Undefined => Value;
     impl< 'a > for &'a mut Undefined => Value;
@@ -495,18 +554,35 @@ impl_infallible_try_from! {
     char => Value;
     impl< 'a > for &'a char => Value;
     impl< 'a > for &'a mut char => Value;
-    impl< T > for Vec< T > => Value where (T: Into< Value >);
-    impl< 'a, T > for &'a Vec< T > => Value where (&'a T: Into< Value >);
-    impl< 'a, T > for &'a mut Vec< T > => Value where (&'a T: Into< Value >);
-    impl< 'a, T > for &'a [T] => Value where (&'a T: Into< Value >);
-    impl< 'a, T > for &'a mut [T] => Value where (&'a T: Into< Value >);
-    impl< K, V > for BTreeMap< K, V > => Value where (K: Into< String >, V: Into< Value >);
-    impl< 'a, K, V > for &'a BTreeMap< K, V > => Value where (&'a K: Into< String >, &'a V: Into< Value >);
-    impl< 'a, K, V > for &'a mut BTreeMap< K, V > => Value where (&'a K: Into< String >, &'a V: Into< Value >);
-    impl< K, V > for HashMap< K, V > => Value where (K: Into< String > + Hash + Eq, V: Into< Value >);
-    impl< 'a, K, V > for &'a HashMap< K, V > => Value where (K: Hash + Eq, &'a K: Into< String >, &'a V: Into< Value >);
-    impl< 'a, K, V > for &'a mut HashMap< K, V > => Value where (K: Hash + Eq, &'a K: Into< String >, &'a V: Into< Value >);
+    impl< T > for Vec< T > => Value where (T: JsSerializable);
+    impl< 'a, T > for &'a Vec< T > => Value where (T: JsSerializable);
+    impl< 'a, T > for &'a mut Vec< T > => Value where (T: JsSerializable);
+    impl< 'a, T > for &'a [T] => Value where (T: JsSerializable);
+    impl< 'a, T > for &'a mut [T] => Value where (T: JsSerializable);
+
+    impl< K, V > for BTreeMap< K, V > => Value where (K: AsRef< str >, V: JsSerializable);
+    impl< 'a, K, V > for &'a BTreeMap< K, V > => Value where (K: AsRef< str >, V: JsSerializable);
+    impl< 'a, K, V > for &'a mut BTreeMap< K, V > => Value where (K: AsRef< str >, V: JsSerializable);
+    impl< K, V > for HashMap< K, V > => Value where (K: AsRef< str > + Eq + Hash, V: JsSerializable);
+    impl< 'a, K, V > for &'a HashMap< K, V > => Value where (K: AsRef< str > + Eq + Hash, V: JsSerializable);
+    impl< 'a, K, V > for &'a mut HashMap< K, V > => Value where (K: AsRef< str > + Eq + Hash, V: JsSerializable);
+
     Reference => Value;
+
+    // TODO: Move these to object.rs
+    impl< K, V > for BTreeMap< K, V > => Object where (K: AsRef< str >, V: JsSerializable);
+    impl< 'a, K, V > for &'a BTreeMap< K, V > => Object where (K: AsRef< str >, V: JsSerializable);
+    impl< 'a, K, V > for &'a mut BTreeMap< K, V > => Object where (K: AsRef< str >, V: JsSerializable);
+    impl< K, V > for HashMap< K, V > => Object where (K: AsRef< str > + Eq + Hash, V: JsSerializable);
+    impl< 'a, K, V > for &'a HashMap< K, V > => Object where (K: AsRef< str > + Eq + Hash, V: JsSerializable);
+    impl< 'a, K, V > for &'a mut HashMap< K, V > => Object where (K: AsRef< str > + Eq + Hash, V: JsSerializable);
+
+    // TODO: Move these to array.rs
+    impl< T > for Vec< T > => Array where (T: JsSerializable);
+    impl< 'a, T > for &'a Vec< T > => Array where (T: JsSerializable);
+    impl< 'a, T > for &'a mut Vec< T > => Array where (T: JsSerializable);
+    impl< 'a, T > for &'a [T] => Array where (T: JsSerializable);
+    impl< 'a, T > for &'a mut [T] => Array where (T: JsSerializable);
 }
 
 macro_rules! impl_try_from_number {
@@ -524,7 +600,7 @@ macro_rules! impl_try_from_number {
     };
 }
 
-impl_try_from_number!( i64 u64 );
+impl_try_from_number!( i64 u64 usize );
 
 impl PartialEq< Undefined > for Value {
     #[inline]
@@ -576,23 +652,6 @@ impl PartialEq< String > for Value {
     }
 }
 
-impl< T > PartialEq< [T] > for Value where Value: PartialEq< T > {
-    #[inline]
-    fn eq( &self, right: &[T] ) -> bool {
-        match *self {
-            Value::Array( ref left ) => left.iter().zip( right.iter() ).all( |(left, right)| left == right ),
-            _ => false
-        }
-    }
-}
-
-impl< 'a, T > PartialEq< &'a [T] > for Value where Value: PartialEq< T > {
-    #[inline]
-    fn eq( &self, right: &&'a [T] ) -> bool {
-        <Self as PartialEq< [T] >>::eq( self, right )
-    }
-}
-
 impl PartialEq< Number > for Value {
     #[inline]
     fn eq( &self, right: &Number ) -> bool {
@@ -703,7 +762,7 @@ macro_rules! impl_partial_eq_to_number {
     };
 }
 
-impl_partial_eq_to_number!( i8 i16 i32 i64 u8 u16 u32 u64 f32 f64 );
+impl_partial_eq_to_number!( i8 i16 i32 i64 u8 u16 u32 u64 usize f32 f64 );
 
 impl_partial_eq_boilerplate! {
     Undefined,
@@ -714,13 +773,16 @@ impl_partial_eq_boilerplate! {
     Number
 }
 
+/// A structure denoting a conversion error encountered when
+/// converting to or from a `Value`.
 #[derive(Clone, PartialEq, Eq, Debug)]
 pub enum ConversionError {
     TypeMismatch {
         actual_type: &'static str
     },
     NumericConversionError( number::ConversionError ),
-    ValueConversionError( Box< ConversionError > )
+    ValueConversionError( Box< ConversionError > ),
+    Custom( String )
 }
 
 fn value_type_name( value: &Value ) -> &'static str {
@@ -741,7 +803,8 @@ impl fmt::Display for ConversionError {
         match *self {
             ConversionError::TypeMismatch { actual_type } => write!( formatter, "type mismatch; actual type is {}", actual_type ),
             ConversionError::NumericConversionError( ref inner ) => write!( formatter, "{}", inner ),
-            ConversionError::ValueConversionError( ref inner ) => write!( formatter, "value conversion error: {}", inner )
+            ConversionError::ValueConversionError( ref inner ) => write!( formatter, "value conversion error: {}", inner ),
+            ConversionError::Custom( ref message ) => write!( formatter, "{}", message )
         }
     }
 }
@@ -751,7 +814,8 @@ impl error::Error for ConversionError {
         match *self {
             ConversionError::TypeMismatch { .. } => "type mismatch",
             ConversionError::NumericConversionError( ref inner ) => inner.description(),
-            ConversionError::ValueConversionError( _ ) => "value conversion error"
+            ConversionError::ValueConversionError( _ ) => "value conversion error",
+            ConversionError::Custom( ref message ) => message
         }
     }
 }
@@ -762,6 +826,12 @@ impl From< number::ConversionError > for ConversionError {
     }
 }
 
+impl From< Void > for ConversionError {
+    fn from( _: Void ) -> Self {
+        unreachable!();
+    }
+}
+
 impl ConversionError {
     #[inline]
     fn type_mismatch( actual_value: &Value ) -> Self {
@@ -771,7 +841,7 @@ impl ConversionError {
     }
 
     #[inline]
-    fn value_conversion_error( inner: ConversionError ) -> Self {
+    pub(crate) fn value_conversion_error( inner: ConversionError ) -> Self {
         ConversionError::ValueConversionError( Box::new( inner ) )
     }
 }
@@ -833,69 +903,39 @@ macro_rules! impl_try_into_number {
     };
 }
 
-impl_try_into_number!( u8 u16 u32 u64 i8 i16 i32 i64 f64 );
+impl_try_into_number!( u8 u16 u32 u64 usize i8 i16 i32 i64 f64 );
 
-impl< V: TryFrom< Value, Error = ConversionError > > TryFrom< Value > for BTreeMap< String, V > {
+impl< E: Into< ConversionError >, V: TryFrom< Value, Error = E > > TryFrom< Value > for BTreeMap< String, V > {
     type Error = ConversionError;
 
     #[inline]
     fn try_from( value: Value ) -> Result< Self, Self::Error > {
         match value {
-            Value::Object( object ) => {
-                let mut output = BTreeMap::new();
-                for (key, value) in object {
-                    let value = match value.try_into() {
-                        Ok( value ) => value,
-                        Err( error ) => return Err( ConversionError::value_conversion_error( error ) )
-                    };
-                    output.insert( key, value );
-                }
-                Ok( output )
-            },
+            Value::Object( object ) => object.try_into(),
             _ => Err( ConversionError::type_mismatch( &value ) )
         }
     }
 }
 
-impl< V: TryFrom< Value, Error = ConversionError > > TryFrom< Value > for HashMap< String, V > {
+impl< E: Into< ConversionError >, V: TryFrom< Value, Error = E > > TryFrom< Value > for HashMap< String, V > {
     type Error = ConversionError;
 
     #[inline]
     fn try_from( value: Value ) -> Result< Self, Self::Error > {
         match value {
-            Value::Object( object ) => {
-                let mut output = HashMap::with_capacity( object.len() );
-                for (key, value) in object {
-                    let value = match value.try_into() {
-                        Ok( value ) => value,
-                        Err( error ) => return Err( ConversionError::value_conversion_error( error ) )
-                    };
-                    output.insert( key, value );
-                }
-                Ok( output )
-            },
+            Value::Object( object ) => object.try_into(),
             _ => Err( ConversionError::type_mismatch( &value ) )
         }
     }
 }
 
-impl< T: TryFrom< Value, Error = ConversionError > > TryFrom< Value > for Vec< T > {
+impl< E: Into< ConversionError >, T: TryFrom< Value, Error = E > > TryFrom< Value > for Vec< T > {
     type Error = ConversionError;
 
     #[inline]
     fn try_from( value: Value ) -> Result< Self, Self::Error > {
         match value {
-            Value::Array( array ) => {
-                let mut output = Vec::with_capacity( array.len() );
-                for value in array {
-                    let value = match value.try_into() {
-                        Ok( value ) => value,
-                        Err( error ) => return Err( ConversionError::value_conversion_error( error ) )
-                    };
-                    output.push( value );
-                }
-                Ok( output )
-            },
+            Value::Array( array ) => array.try_into(),
             _ => Err( ConversionError::type_mismatch( &value ) )
         }
     }
@@ -991,6 +1031,7 @@ impl_nullable_try_from_value! {
     u16;
     u32;
     u64;
+    usize;
     i8;
     i16;
     i32;
@@ -1051,14 +1092,6 @@ mod tests {
         assert!( value != "Bob" );
     }
 
-    #[test]
-    fn array_equality() {
-        let value = Value::Array( vec![ Value::Bool( true ), Value::Bool( false ) ] );
-        assert!( value == &[true, false][..] );
-        assert!( value != &[true, true][..] );
-        // Looks like it's not possible to define a symmetric PartialEq for arrays. );
-    }
-
     #[test]
     fn reference_equality() {
         let value = js! { return new Date() };
